#include "displaynmcontain.h"
#include <QScrollBar>

DisplayNMContain::DisplayNMContain(QWidget *parent) : QScrollArea(parent)
{
    this->setObjectName("DisplayNMContain");
    this->setFocusPolicy(Qt::NoFocus);
    this->setFrameShape(QFrame::NoFrame);
    this->setFrameShadow(QFrame::Plain);
    this->setLineWidth(0);
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setWidgetResizable(true);

    m_wgtContain = new QWidget(this);
    m_wgtContain->setObjectName("DisplayNMContainWidget");
    m_layoutContain = new QVBoxLayout;
    m_layoutContain->setMargin(0);
    m_layoutContain->setContentsMargins(0,0,0,0);
    m_layoutContain->setSpacing(0);
    m_layoutContain->addStretch();
    m_wgtContain->setLayout(m_layoutContain);
    m_wgtContain->setMinimumSize(this->size());
    this->setWidget(m_wgtContain);

    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionAdded, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionRemoved, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionsChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activatingConnectionChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::primaryConnectionChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::statusChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::settingsNotifier(), &NetworkManager::SettingsNotifier::connectionAdded, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(NetworkManager::settingsNotifier(), &NetworkManager::SettingsNotifier::connectionRemoved, this, &DisplayNMContain::onActiveConnectionsChanged);

    //手动设置一下网络为可用状态
    NetworkManager::setNetworkingEnabled(true);

    m_cableNetworkflag=false;

    m_wifiFlag=false;
    m_timer = new QTimer(this);
    m_timer->setInterval(2 * 1000);
    connect(m_timer, &QTimer::timeout, this, &DisplayNMContain::onTimeout);
}

void DisplayNMContain::addDevice(NetworkManager::Device::Ptr device)
{


    mDevices.append(device);
    connect(device.data(), &NetworkManager::Device::stateChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::activeConnectionChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::autoconnectChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::availableConnectionChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::availableConnectionAppeared, this, &DisplayNMContain::onConnectionAdded);
    connect(device.data(), &NetworkManager::Device::availableConnectionDisappeared, this, &DisplayNMContain::onConnectionRemoved);
    connect(device.data(), &NetworkManager::Device::capabilitiesChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::dhcp4ConfigChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::dhcp6ConfigChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::driverChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::driverVersionChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::firmwareMissingChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::firmwareVersionChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::interfaceNameChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV4AddressChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV4ConfigChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV6ConfigChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipInterfaceChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::managedChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::physicalPortIdChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::mtuChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::nmPluginMissingChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::meteredChanged, this, &DisplayNMContain::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::connectionStateChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::stateReasonChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::udiChanged, this, &DisplayNMContain::onDeviceUpdated);

    NetworkManager::WiredDevice * wiredDevice = qobject_cast<NetworkManager::WiredDevice *>(device.data());
    if(wiredDevice)
    {
        connect(wiredDevice, &NetworkManager::WiredDevice::bitRateChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wiredDevice, &NetworkManager::WiredDevice::carrierChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wiredDevice, &NetworkManager::WiredDevice::hardwareAddressChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wiredDevice, &NetworkManager::WiredDevice::permanentHardwareAddressChanged, this, &DisplayNMContain::onDeviceUpdated);

        //获取可用的有线连接
        for (auto const & conn : wiredDevice->availableConnections())
        {
            DisplayWiredItem * item = new DisplayWiredItem(wiredDevice, conn, this);
            connect(item, &DisplayWiredItem::signalItemClicked, this, &DisplayNMContain::onItemClicked);
            connect(item, &DisplayWiredItem::signalItemHeightChange, this, &DisplayNMContain::onItemHeightChange);
            m_layoutContain->insertWidget(m_layoutContain->count()-1, item);
            m_itemWires.append(item);
            m_cableNetworkflag=true;
        }
    }


    NetworkManager::WirelessDevice * wirelessDevice = qobject_cast<NetworkManager::WirelessDevice *>(device.data());
    if(wirelessDevice)
    {
        connect(wirelessDevice, &NetworkManager::WirelessDevice::bitRateChanged, this, &DisplayNMContain::onActiveConnectionsChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::activeAccessPointChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::modeChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessCapabilitiesChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::hardwareAddressChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::permanentHardwareAddressChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessPropertiesChanged, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointAppeared, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointDisappeared, this, &DisplayNMContain::onDeviceUpdated);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::networkAppeared, this, &DisplayNMContain::onWifiNetworkAppeared);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::networkDisappeared, this, &DisplayNMContain::onWifiNetworkDisappeared);

        for(auto const & network : wirelessDevice->networks())
        {
            qWarning() << network->ssid();
            DisplayWifiItem * item = new DisplayWifiItem(wirelessDevice, network, this);
            connect(item, &DisplayWifiItem::signalItemClicked, this, &DisplayNMContain::onItemClicked);
            connect(item, &DisplayWifiItem::signalItemHeightChange, this, &DisplayNMContain::onItemHeightChange);
            m_layoutContain->insertWidget(m_layoutContain->count() - 1, item);
            m_itemWifis.append(item);
            m_wifiFlag=true;
        }
    }

    resetContainHeight();
    onActiveConnectionsChanged();
}

void DisplayNMContain::removeDevice(NetworkManager::Device::Ptr device)
{

    if(mDevices.contains(device))
    {
        mDevices.removeAll(device);
    }
}

void DisplayNMContain::onActiveConnectionsChanged()
{
    for(DisplayWifiItem * item : m_itemWifis)
    {
        item->updateActiveStatu();
        if(item->getActiveStatu())
        {
            m_layoutContain->removeWidget(item);
            int index = getIndexMidle();
            m_layoutContain->insertWidget(index, item);
        }
    }
    for(DisplayWiredItem * item : m_itemWires)
    {
        item->updateActiveStatu();
        if(item->getActiveStatu())
        {
            m_layoutContain->removeWidget(item);
            m_layoutContain->insertWidget(0, item);
        }
    }
}

void DisplayNMContain::onConnectionAdded(const QString &path)
{
    //新建有线连接
    NetworkManager::Connection::Ptr addConn = NetworkManager::findConnection(path);

    //每次删除连接时，
    if(addConn && addConn->isValid() && !addConn->name().isEmpty())
    {
        //查找该连接所在的设备
        NetworkManager::WiredDevice * wiredDevice = nullptr;
        for(NetworkManager::Device::Ptr device : mDevices)
        {
            NetworkManager::WiredDevice * d = qobject_cast<NetworkManager::WiredDevice *>(device.data());
            if(d)
            {
                for (NetworkManager::Connection::Ptr const & dev_conn : d->availableConnections())
                {
                    qWarning() << dev_conn->path() << "-----" << addConn->path();
                    if (dev_conn == addConn)
                    {
                        wiredDevice = d;
                        break;
                    }
                }
            }
            if(wiredDevice) break;
        }
        if(wiredDevice)
        {
            qWarning() << "新建有线连接" << addConn->name();
            DisplayWiredItem * item = new DisplayWiredItem(wiredDevice, addConn, this);
            connect(item, &DisplayWiredItem::signalItemClicked, this, &DisplayNMContain::onItemClicked);
            //这里需要特殊处理一下，新加的有线连接添加到连接后面
            int index = getIndexMidle();
            m_layoutContain->insertWidget(index, item);
            m_itemWires.append(item);
            resetContainHeight();

            m_cableNetworkflag=true;
        }
    }
}

void DisplayNMContain::onConnectionRemoved(const QString &path)
{
    for(DisplayWiredItem * item : m_itemWires)
    {
        qWarning() << "删除连接----" << item->getConnect()->path() << "   " << path;
        if(item->getConnect()->path() == path)
        {
            qWarning() << "删除连接" << item->getConnect()->name();
            m_itemWires.removeAll(item);
            m_layoutContain->removeWidget(item);
            resetContainHeight();
            delete item;
             m_cableNetworkflag=false;
            break;
        }
    }
}

void DisplayNMContain::onDeviceUpdated()
{
//
}

void DisplayNMContain::onWifiNetworkAppeared(const QString &ssid)
{
    for(NetworkManager::Device::Ptr device : mDevices)
    {
        NetworkManager::WirelessDevice * wirelessDevice = qobject_cast<NetworkManager::WirelessDevice *>(device.data());
        if(wirelessDevice)
        {
            NetworkManager::WirelessNetwork::Ptr network = wirelessDevice->findNetwork(ssid);
            if(!network.isNull())
            {
                qWarning() << "WIFI新建连接" << network->ssid();
                DisplayWifiItem * item = new DisplayWifiItem(wirelessDevice, network, this);
                connect(item, &DisplayWifiItem::signalItemClicked, this, &DisplayNMContain::onItemClicked);
                connect(item, &DisplayWifiItem::signalItemHeightChange, this, &DisplayNMContain::onItemHeightChange);
                int index = -1;
                for(int i = 0; i < m_itemWifis.size(); i++)
                {
                    if(!m_itemWifis[i]->getActiveStatu()
                            && QString::compare(network->ssid().toLower(), m_itemWifis[i]->getNetwork()->ssid().toLower()) <=0)
                        index = i;
                }
                if(index == -1)
                {
                    m_layoutContain->insertWidget(m_layoutContain->count() - 1, item);
                    m_itemWifis.push_front(item);
                }
                else{
                    m_itemWifis.insert(index + 1, item);
                    m_layoutContain->insertWidget(m_itemWifis.count() - index - 2 + getIndexMidle(), item);
                }
                for(auto it = m_toDelItemWifis.begin(); it != m_toDelItemWifis.end(); it++)
                {
                    if((*it)->getNetwork()->ssid() == item->getNetwork()->ssid())
                    {
                        m_layoutContain->removeWidget(*it);
                        m_itemWifis.removeOne(*it);
                        (*it)->deleteLater();
                        *it = nullptr;
                        m_toDelItemWifis.erase(it);
                        break;
                    }
                }
//                m_layoutContain->insertWidget(m_layoutContain->count() - 1, item);
//                m_itemWifis.append(item);
                resetContainHeight();
                onActiveConnectionsChanged();
                 m_wifiFlag=true;
                break;
            }
        }
    }
}

void DisplayNMContain::onWifiNetworkDisappeared(const QString &ssid)
{
    for(DisplayWifiItem * item : m_itemWifis)
    {
        if(item->getNetwork()->ssid() == ssid)
        {
            qWarning() << "WIFI删除连接" << item->getNetwork()->ssid();
            m_toDelItemWifis.append(item);
//            m_layoutContain->removeWidget(item);
//            m_itemWifis.removeAll(item);
//            resetContainHeight();
//            delete item;
            break;
        }
    }
    if(m_timer->isActive())
        m_timer->stop();
    m_timer->start();
}

void DisplayNMContain::onItemClicked()
{
    QObject * objSender = sender();
    for(DisplayWifiItem * item : m_itemWifis)
    {
        if(objSender != item)
        {
            item->otherItemClicked();
        }
    }
    for(DisplayWiredItem * item : m_itemWires)
    {
        if(item != objSender)
        {
            item->updateActiveStatu();
        }
    }
    this->resetContainHeight();
}

void DisplayNMContain::onItemHeightChange(int itemHeight)
{
    this->resetContainHeight();
}

void DisplayNMContain::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void DisplayNMContain::mouseReleaseEvent(QMouseEvent *event)
{
    onItemClicked();
    QWidget::mouseReleaseEvent(event);
}

void DisplayNMContain::showEvent(QShowEvent *event)
{
    this->verticalScrollBar()->setValue(0);
    sortList(SortByName);
    onActiveConnectionsChanged();
}

void DisplayNMContain::resetContainHeight()
{
    int height = 0;
    for(DisplayWifiItem * item : m_itemWifis)
    {
        height += item->height();
    }
    for(DisplayWiredItem * item : m_itemWires)
    {
        height += item->height();
    }

    height = qMax(this->height(), height);
    m_wgtContain->setMinimumSize(this->width(), height);
}

int DisplayNMContain::getIndexMidle()
{
    //这里需要特殊处理一下，新加的有线连接添加到连接后面
    int index = -1;
    for(DisplayWiredItem * tmpItem : m_itemWires)
    {
        if(index < m_layoutContain->indexOf(tmpItem))
        {
            index = m_layoutContain->indexOf(tmpItem);
        }
    }
    return index+1;
}

void DisplayNMContain::sortList(DisplayNMContain::SortRole role)
{
    switch (role) {
    case SortByName:
        qSort(m_itemWifis.begin(), m_itemWifis.end(), [](DisplayWifiItem*a, DisplayWifiItem* b)->bool{
            if(a->getActiveStatu()) return false;
            else if(b->getActiveStatu()) return true;
            else
            {
                int ret = QString::compare(a->getNetwork()->ssid().toLower(), b->getNetwork()->ssid().toLower());
                return ret <= 0 ? false : true;
            }
        });
        break;
    case SortBySignal:
        qSort(m_itemWifis.begin(), m_itemWifis.end(), [](DisplayWifiItem*a, DisplayWifiItem* b)->bool{
            if(a->getActiveStatu()) return false;
            else if(b->getActiveStatu()) return true;
            else return a->getNetwork()->signalStrength() < b->getNetwork()->signalStrength();
        });
        break;
    }
    QList<DisplayWifiItem*>::iterator it = m_itemWifis.begin();
    int index = getIndexMidle();
    for(it; it != m_itemWifis.end(); it++)
    {
        m_layoutContain->removeWidget(*it);
        m_layoutContain->insertWidget(index, *it);
    }
    m_layoutContain->update();
}

void DisplayNMContain::onTimeout()
{
    for(auto it = m_itemWifis.begin(); it != m_itemWifis.end(); it++)
    {
        //如果正在连接，不刷新列表
        if((*it)->getActivingStatu())
            return;
    }
    //正在断开连接，不刷新列表,如果10秒内搜索不到新WiFi，刷新列表
    static int count = 0;
    if(m_itemWifis.size() == m_toDelItemWifis.size()
            && NetworkManager::isWirelessEnabled()
            && !m_itemWifis.isEmpty())
    {
        if(count < 5)
        {
            count++;
            return;
        }
    }
    count = 0;
    if(m_timer->isActive())
        m_timer->stop();
    for(auto it = m_toDelItemWifis.begin(); it != m_toDelItemWifis.end(); it++)
    {
        m_layoutContain->removeWidget(*it);
        m_itemWifis.removeOne((*it));
        (*it)->deleteLater();
        *it = nullptr;
    }
    m_toDelItemWifis.clear();
    resetContainHeight();
}
