#ifndef DISPLAYNMCONTAIN_H
#define DISPLAYNMCONTAIN_H

#include <QScrollArea>
#include <NetworkManagerQt/WirelessDevice>
#include <NetworkManagerQt/Device>
#include <NetworkManagerQt/ConnectionSettings>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <QStyleOption>
#include <QPainter>
#include <QLabel>
#include <QBoxLayout>
#include "displaywifiitem.h"
#include "displaywireditem.h"

class DisplayNMContain : public QScrollArea
{
    Q_OBJECT
    typedef  enum SortRole
    {
        SortByName,
        SortBySignal
    }SortRole;
public:
    explicit DisplayNMContain(QWidget *parent = nullptr);

    void addDevice(NetworkManager::Device::Ptr device);
    void removeDevice(NetworkManager::Device::Ptr device);
    bool m_cableNetworkflag;
    bool m_wifiFlag;

Q_SIGNALS:
    void signalNotify(QString wifiName);

public Q_SLOTS:
    //关联NetWorkManager 信号的槽
    void onActiveConnectionsChanged();
    void onConnectionAdded(const QString &path);
    void onConnectionRemoved(const QString &path);

    //关联WIFI Device信号的槽
    void onDeviceUpdated();
    void onWifiNetworkAppeared(QString const & ssid);
    void onWifiNetworkDisappeared(QString const & ssid);

    //关联item信号的槽
    void onItemClicked();
    void onItemHeightChange(int itemHeight);

protected:
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void showEvent(QShowEvent* event);

private:
    void resetContainHeight();
    //获取wifi和有线连接m_layoutContain中间的index
    int getIndexMidle();
    void sortList(SortRole role);

private slots:
    void onTimeout();
private:
    QVBoxLayout * m_layoutContain;
    QWidget * m_wgtContain;
    QList<DisplayWifiItem*> m_itemWifis;
    QList<DisplayWifiItem*> m_toDelItemWifis;
    QList<DisplayWiredItem*> m_itemWires;
    QTimer* m_timer;

    //无线设备列表
    NetworkManager::Device::List mDevices;
};

#endif // DISPLAYNMCONTAIN_H
