#ifndef DISPLAYWIFIITEM_H
#define DISPLAYWIFIITEM_H

#include <QWidget>
#include <nde/Nde/Settings>
#include <NetworkManagerQt/WirelessNetwork>
#include <NetworkManagerQt/Manager>
#include <QPropertyAnimation>
#include <QTimer>

namespace Ui {
class DisplayWifiItem;
}

class DisplayWifiItem : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(int itemHeight READ itemHeight WRITE setItemHeight)
    enum ENUM_ShowType{
        ShowInit,
        ShowStatus,    //default
        ShowLink,
        ShowInputKey,
        ShowLinking,
        ShowLinkFinished
    };

public:
    explicit DisplayWifiItem(NetworkManager::WirelessDevice * device, NetworkManager::WirelessNetwork::Ptr network, QWidget *parent = nullptr);
    ~DisplayWifiItem();

    NetworkManager::WirelessNetwork::Ptr getNetwork() const;

    //更新该连接的激活状态
    void updateActiveStatu();
    bool getActiveStatu();
    bool getActivingStatu();
    void otherItemClicked();

    //property
    int itemHeight();
    void setItemHeight(int height) ;

Q_SIGNALS:
    void signalItemClicked();
    void signalItemHeightChange(int height);

private Q_SLOTS:
    void onSignalStrengthChanged(int strength);
    void onReferenceAccessPointChanged(const QString &apUni);
    void onDisappeared(const QString &ssid);

private Q_SLOTS:
    void onLinkAuto();
    void onLinkLink();
    void onLinkForget();
    void onLinkBreak();
    void onInputKeyChange();
    void onInputKeyOK();
    void onInputKeyCancel();
    void onNoKeyOK();
    void onNoKeyCancel();
    void onLinkingCancel();
    void onLinkFinishedClose();

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);

private:
    void showCurrentLinkStatus(ENUM_ShowType type);
    void showStatus();
    void showLink();
    void showInputKey();
    void showLinking();
    void showFinished();

    //计算item的高度
    int calcItemHeight();

    //检查是否激活
    NetworkManager::ActiveConnection::State checkActive();
    //检查是否是已知
    bool checkKnown();
    //检查是否是自动连接
    bool checkAuto();
    //检查该wifi是否需要密码，返回true：password为密码，返回false：password为空
    bool checkNeedPassword(QString & password);
    //获取该wifi建立的连接
    NetworkManager::Connection::Ptr getConnect();

private:
    Ui::DisplayWifiItem *ui;

    ENUM_ShowType m_showType;
    NetworkManager::ActiveConnection::State m_active;  //激活状态
    bool m_known;   //记住
    bool m_auto;    //自动连接
    bool m_needPassword; //该wifi是否需要密码

    NetworkManager::WirelessDevice * m_wirelessDevice;
    NetworkManager::WirelessNetwork::Ptr m_wirelessNetwork;

    int mConnectingTimerCount; //动画计数
    QTimer *mConnectingTimer; //连接中动画
    QPropertyAnimation *m_resizeAnimation; //高度变化的动画效果
};

#endif // DISPLAYWIFIITEM_H
