#ifndef DISPLAYWIREDITEM_H
#define DISPLAYWIREDITEM_H

#include <QWidget>
#include <QLabel>
#include <QToolButton>
#include <QBoxLayout>
#include <NetworkManagerQt/Connection>
#include <NetworkManagerQt/WiredDevice>
#include <QPropertyAnimation>

class DisplayWiredItem : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(int itemHeight READ itemHeight WRITE setItemHeight)
    enum ENUM_ShowType{
        ShowInit,
        ShowStatus, //default
        ShowLink
    };
public:
    explicit DisplayWiredItem(NetworkManager::WiredDevice * device, NetworkManager::Connection::Ptr conn, QWidget *parent = nullptr);
    virtual ~DisplayWiredItem();

    NetworkManager::Connection::Ptr getConnect() const;

    //更新该连接的激活状态
    void updateActiveStatu();

    bool getActiveStatu();

    //property
    int itemHeight();
    void setItemHeight(int height) ;

Q_SIGNALS:
    void signalItemClicked();
    void signalItemHeightChange(int height);

public Q_SLOTS:

private Q_SLOTS:
    void onBtnClicked();
    void onConnectUpdated();
    void onConnectUnsavedChanged(bool unsaved);

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);

private:
    void onShowStatus();
    void onShowLink();
    bool checkActive();

private:
    ENUM_ShowType m_showType;    //显示类型
    bool m_active;
    QLabel * m_labelIcon;
    QLabel * m_labelName;
    QLabel * m_labelStatus;
    QToolButton * m_btn;

    NetworkManager::WiredDevice * m_wiredDevice;
    NetworkManager::Connection::Ptr m_wiredConnect;

    QPropertyAnimation *m_resizeAnimation; //高度变化的动画效果
};

#endif // DISPLAYWIREDITEM_H
